/*
 * neopixel mqtt stop light, D5 as signal for three (3) ws2812s
 * stop light changes states via MQTT commands
 * 
 * commands sent via MQTT topic
 * default state = off
 * 0 = off
 * 1 = red
 * 2 = yellow
 * 3 = green
 * 
 */

#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Adafruit_NeoPixel.h>

#define PIN D5 // neopixel

// ipdate these values for your network
const char* ssid = "your_ssid";
const char* password = "your_passwd";
const char* mqtt_server = "your_server";
const char* subTopic = "stoplight/";

// Parameter 1 = number of pixels in strip
// Parameter 2 = pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(3, PIN, NEO_RGB + NEO_KHZ800);

// define the color codes by name to make it easier to call later on
uint32_t green = strip.Color(255, 0, 0);
uint32_t red = strip.Color(0, 255, 0);
uint32_t yellow = strip.Color(255, 100, 0);

WiFiClient espClient;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;

void setup_wifi() {
  delay(100);
  // we start by connecting to a WiFi network
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
    Serial.print(".");
  }
  randomSeed(micros());
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payload, unsigned int length) 
{
  Serial.print("Command from MQTT broker is : [");
  Serial.print(topic);
  int p =(char)payload[0]-'0';

  switch (p) {
    case 0:
      Serial.println("turn off");
      strip.clear();
      strip.show();
      break;
    case 1:
      Serial.println("red");
      strip.clear();
      strip.setPixelColor(0, red);
      strip.show();
      break;
    case 2:
      Serial.println("yellow");
      strip.clear();
      strip.setPixelColor(1, yellow);
      strip.show();
      break;
    case 3:
      Serial.println("green");
      strip.clear();
      strip.setPixelColor(2, green);
      strip.show();
      break;
    default:
      Serial.println("off");
      strip.clear();
      strip.show();
  }
  
  Serial.println();
} // end callback

void reconnect() {
  // loop until we're reconnected
  while (!client.connected()) 
  {
    Serial.print("Attempting MQTT connection...");
    // create a random client ID
    String clientId = "ESP8266Client-";
    clientId += String(random(0xffff), HEX);
    // attempt to connect
    // if you MQTT broker has clientID, username and password
    // please change following line to "if (client.connect(clientId,userName,passWord))"
    if (client.connect(clientId.c_str()))
    {
      Serial.println("connected");
     // once connected to MQTT broker, subscribe command if any
      client.subscribe(subTopic);
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // wait 6 seconds before retrying
      delay(6000);
    }
  }
} // end reconnect()

void setup() {
  Serial.begin(115200);
  
  strip.begin();
  strip.show();   // initialize all pixels to 'off'
  
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void loop() {
  if (!client.connected()) {
    reconnect();
  }
  client.loop();
}


